@@ -1,7 +1,7 @@
 /*
  * This file is part of the trojan project.
  * Trojan is an unidentifiable mechanism that helps you bypass GFW.
- * Copyright (C) 2017-2019  GreaterFire
+ * Copyright (C) 2017-2019  GreaterFire, wongsyrone
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,20 +24,25 @@ using namespace std;
 using namespace boost::asio::ip;
 using namespace boost::asio::ssl;
 
-ForwardSession::ForwardSession(const Config &config, boost::asio::io_service &io_service, context &ssl_context) :
-    Session(config, io_service),
+ForwardSession::ForwardSession(const Config &config, boost::asio::io_context &io_context, context &ssl_context) :
+    Session(config, io_context),
     status(CONNECT),
     first_packet_recv(false),
-    in_socket(io_service),
-    out_socket(io_service, ssl_context) {}
+    in_socket(io_context),
+    out_socket(io_context, ssl_context) {}
 
 tcp::socket& ForwardSession::accept_socket() {
     return in_socket;
 }
 
 void ForwardSession::start() {
+    boost::system::error_code ec;
     start_time = time(NULL);
-    in_endpoint = in_socket.remote_endpoint();
+    in_endpoint = in_socket.remote_endpoint(ec);
+    if (ec) {
+        destroy();
+        return;
+    }
     auto ssl = out_socket.native_handle();
     if (config.ssl.sni != "") {
         SSL_set_tlsext_host_name(ssl, config.ssl.sni.c_str());
@@ -48,36 +53,37 @@ void ForwardSession::start() {
             SSL_set_session(ssl, session);
         }
     }
-    out_write_buf = TrojanRequest::generate(config.password.cbegin()->first, config.target_addr, config.target_port);
-    if (config.append_payload) {
-        in_async_read();
-    } else {
-        first_packet_recv = true;
-    }
+    out_write_buf = TrojanRequest::generate(config.password.cbegin()->first, config.target_addr, config.target_port, true);
+    in_async_read();
     Log::log_with_endpoint(in_endpoint, "forwarding to " + config.target_addr + ':' + to_string(config.target_port) + " via " + config.remote_addr + ':' + to_string(config.remote_port), Log::INFO);
-    tcp::resolver::query query(config.remote_addr, to_string(config.remote_port));
     auto self = shared_from_this();
-    resolver.async_resolve(query, [this, self](const boost::system::error_code error, tcp::resolver::iterator iterator) {
+    resolver.async_resolve(config.remote_addr, to_string(config.remote_port), [this, self](const boost::system::error_code error, tcp::resolver::results_type results) {
         if (error) {
             Log::log_with_endpoint(in_endpoint, "cannot resolve remote server hostname " + config.remote_addr + ": " + error.message(), Log::ERROR);
             destroy();
             return;
         }
-        out_socket.lowest_layer().open(iterator->endpoint().protocol());
+        auto iterator = results.begin();
+        boost::system::error_code ec;
+        out_socket.next_layer().open(iterator->endpoint().protocol(), ec);
+        if (ec) {
+            destroy();
+            return;
+        }
         if (config.tcp.no_delay) {
-            out_socket.lowest_layer().set_option(tcp::no_delay(true));
+            out_socket.next_layer().set_option(tcp::no_delay(true));
         }
         if (config.tcp.keep_alive) {
-            out_socket.lowest_layer().set_option(boost::asio::socket_base::keep_alive(true));
+            out_socket.next_layer().set_option(boost::asio::socket_base::keep_alive(true));
         }
 #ifdef TCP_FASTOPEN_CONNECT
         if (config.tcp.fast_open) {
             using fastopen_connect = boost::asio::detail::socket_option::boolean<IPPROTO_TCP, TCP_FASTOPEN_CONNECT>;
             boost::system::error_code ec;
-            out_socket.lowest_layer().set_option(fastopen_connect(true), ec);
+            out_socket.next_layer().set_option(fastopen_connect(true), ec);
         }
 #endif // TCP_FASTOPEN_CONNECT
-        out_socket.lowest_layer().async_connect(*iterator, [this, self](const boost::system::error_code error) {
+        out_socket.next_layer().async_connect(*iterator, [this, self](const boost::system::error_code error) {
             if (error) {
                 Log::log_with_endpoint(in_endpoint, "cannot establish connection to remote server " + config.remote_addr + ':' + to_string(config.remote_port) + ": " + error.message(), Log::ERROR);
                 destroy();
@@ -113,7 +119,10 @@ void ForwardSession::start() {
 void ForwardSession::in_async_read() {
     auto self = shared_from_this();
     in_socket.async_read_some(boost::asio::buffer(in_read_buf, MAX_LENGTH), [this, self](const boost::system::error_code error, size_t length) {
-        if (error && error != boost::asio::error::operation_aborted) {
+        if (error == boost::asio::error::operation_aborted) {
+            return;
+        }
+        if (error) {
             destroy();
             return;
         }
@@ -123,7 +132,8 @@ void ForwardSession::in_async_read() {
 
 void ForwardSession::in_async_write(const string &data) {
     auto self = shared_from_this();
-    boost::asio::async_write(in_socket, boost::asio::buffer(data), [this, self](const boost::system::error_code error, size_t) {
+    auto data_copy = make_shared<string>(data);
+    boost::asio::async_write(in_socket, boost::asio::buffer(*data_copy), [this, self, data_copy](const boost::system::error_code error, size_t) {
         if (error) {
             destroy();
             return;
@@ -145,7 +155,8 @@ void ForwardSession::out_async_read() {
 
 void ForwardSession::out_async_write(const string &data) {
     auto self = shared_from_this();
-    boost::asio::async_write(out_socket, boost::asio::buffer(data), [this, self](const boost::system::error_code error, size_t) {
+    auto data_copy = make_shared<string>(data);
+    boost::asio::async_write(out_socket, boost::asio::buffer(*data_copy), [this, self, data_copy](const boost::system::error_code error, size_t) {
         if (error) {
             destroy();
             return;
@@ -197,13 +208,21 @@ void ForwardSession::destroy() {
         in_socket.shutdown(tcp::socket::shutdown_both, ec);
         in_socket.close(ec);
     }
-    if (out_socket.lowest_layer().is_open()) {
-        out_socket.lowest_layer().cancel(ec);
+    if (out_socket.next_layer().is_open()) {
         auto self = shared_from_this();
-        out_socket.async_shutdown([this, self](const boost::system::error_code) {
+        auto ssl_shutdown_cb = [this, self](const boost::system::error_code error) {
+            if (error == boost::asio::error::operation_aborted) {
+                return;
+            }
             boost::system::error_code ec;
-            out_socket.lowest_layer().shutdown(tcp::socket::shutdown_both, ec);
-            out_socket.lowest_layer().close(ec);
-        });
+            ssl_shutdown_timer.cancel();
+            out_socket.next_layer().cancel(ec);
+            out_socket.next_layer().shutdown(tcp::socket::shutdown_both, ec);
+            out_socket.next_layer().close(ec);
+        };
+        out_socket.next_layer().cancel(ec);
+        out_socket.async_shutdown(ssl_shutdown_cb);
+        ssl_shutdown_timer.expires_after(chrono::seconds(SSL_SHUTDOWN_TIMEOUT));
+        ssl_shutdown_timer.async_wait(ssl_shutdown_cb);
     }
 }
