@@ -20,23 +20,36 @@
 #ifndef _SERVICE_H_
 #define _SERVICE_H_
 
-#include <boost/asio/io_service.hpp>
+#include <list>
+#include <boost/asio/io_context.hpp>
 #include <boost/asio/ssl.hpp>
+#include <boost/asio/ip/udp.hpp>
 #include "authenticator.h"
+#include "udpforwardsession.h"
 
 class Service {
 private:
+    enum {
+        MAX_LENGTH = 8192
+    };
     const Config &config;
-    boost::asio::io_service io_service;
+    boost::asio::io_context io_context;
     boost::asio::ip::tcp::acceptor socket_acceptor;
     boost::asio::ssl::context ssl_context;
     Authenticator *auth;
     std::string plain_http_response;
+    boost::asio::ip::udp::socket udp_socket;
+    std::list<std::weak_ptr<UDPForwardSession> > udp_sessions;
+    uint8_t udp_read_buf[MAX_LENGTH];
+    boost::asio::ip::udp::endpoint udp_recv_endpoint;
     void async_accept();
+    void udp_async_read();
 public:
     Service(Config &config, bool test = false);
     void run();
     void stop();
+    boost::asio::io_context &service();
+    void reload_cert();
     ~Service();
 };
 
